//
//  7segLED 単純固定値表示 サンプルプログラム
//    copyright (c) 2013 T.Nakao      http://www.wsnak.com
//
//  12/06/09
//
//  p2_7_7seg

#define OUT_SEGDAT PORTD        // パターンデータ出力ポート

// カラム切り替え("L"で点灯)
#define SEL_COL1 PORTB &=~ (1<<0)    // D8(PB0)  1の桁 ON
#define SEL_COL2 PORTB &=~ (1<<1)    // D9(PB1)  10の桁 ON
#define SEL_COL3 PORTB &=~ (1<<2)    // D10(PB2) 100の桁 ON
#define SEL_COL4 PORTB &=~ (1<<3)    // D11(PB3) 1000の桁 ON

#define UNSEL_ALL PORTB |= (0x0F)   // 全桁OFF

byte ColNum;    // カラム番号(n)  0～3

byte BcdVal[4] = {4, 3, 2, 1};    // 表示する数値 "1234" 最下位桁から

// セグメントパターン LSBがセグメント"a"
byte SegPat[10] = {
  0x3F,   // 00111111 数字"0"のパターン
  0x06,   // 00000110 数字"1"のパターン
  0x5B,   // 01011011 数字"2"のパターン
  0x4F,   // 01001111 数字"3"のパターン
  0x66,   // 01100110 数字"4"のパターン
  0x6D,   // 01101101 数字"5"のパターン
  0x7D,   // 01111101 数字"6"のパターン
  0x07,   // 00000111 数字"7"のパターン
  0x7F,   // 01111111 数字"8"のパターン
  0x67    // 01100111 数字"9"のパターン
};


// 初期化
void setup(void) {
  // D0～D11を出力に設定
  DDRD = 0xFF;  // D0-D7 output
  DDRB |= 0x0F; // D8-D11 output
  UNSEL_ALL;    // 全桁OFF

  ColNum = 0;   // 1の桁から始める
}


// メインループ
void loop(void) {
  byte pat;       // セグメント表示(パターン)データ
  byte ix;        // セグメントパターン配列のインデックス

  ix = BcdVal[ColNum];  // 表示桁の数字を取り出す。
  pat = SegPat[ix];   // 数字から表示パターンを得る

  OUT_SEGDAT = pat;   // パターン(セグメントデータ)の出力

  // 表示区間の始まり
  switch(ColNum) {
    case 0:
      SEL_COL1;   // 1の桁をON
      break;
    case 1:
      SEL_COL2;   // 10の桁をON
      break;
    case 2:
      SEL_COL3;   // 100の桁をON
      break;
    case 3:
      SEL_COL4;   // 1000の桁をON
      break;
  }
  //delay(500);         // 500ms
  delayMicroseconds(500);   // 500us

  // 全桁非表示区間
  UNSEL_ALL;          // いったん全桁OFF
  delayMicroseconds(100);   // 50us

  ColNum++;          // 次の桁番号に設定
  if(ColNum > 3) {
    ColNum = 0;       // 全桁の表示終わったとき、最初の桁に戻す
  }
}
