//
//  CANノードA 温度測定
//    copyright (c) 2013 T.Nakao      http://www.wsnak.com
//    13/05/30 関数名 誤記訂正
//

#include <wCan2515.h>
#include <wCTimer.h>
#include <p2515Reg.h>

// CANボードのLED点消灯マクロ
#define CANLED_ON Can.BitModCmd(BFPCTRL, (1<<B0BFS) | (1<<B0BFE), (1<<B0BFS) | (1<<B0BFE))
#define CANLED_OFF  Can.BitModCmd(BFPCTRL, (1<<B0BFS) | (1<<B0BFE), 1<<B0BFE)

wCan2515 Can(2, 4, 3, 5, CAN_BRP_16MHz_125KBPS);  // CANオブジェクト
// D2:SCK, D3:SO, D4:SI, D5:SS (SI/SOクロス)

wCtcTimer2A tm;                       // タイマオブジェクト

byte CanBuf[14];    // CAN送受信バッファ
int LedPin = 13;    // オンボードLEDのポート番号
byte Count = 0;
byte LedSts = true;


// 初期化
void setup(void) {
  pinMode(LedPin, OUTPUT);
  tm.init(125);               // タイマ周期を8msに設定 130530

  Can.SetMask(MASK_SID_ALL_HIT);
  Can.SetFilter(0, FILTER_MOD_ALL_HIT); // すべてのメッセージを受信
  Can.SetOpMode(CAM_MODE_NORMAL);

  // SPI送信確認 CANボードのLED点灯
  CANLED_ON;    // LED ON
  delay(500);
  CANLED_OFF;   // LED OFF
  delay(500);
  CANLED_ON;    // LED ON
}


// メインループ
void loop(void) {
  byte size, datfrm;
  unsigned int msg;
  byte dat[8];

  if(Can.RxCheck()) {
    // 受信しているとき
    msg = Can.GetRxBuf(0, dat, &size, &datfrm);   // データとメッセージを取り出す。
    if(msg == 0x0101) {
      // LED ON
      digitalWrite(LedPin, HIGH);
    } else if(msg == 0x0100) {
      // LED OFF
      digitalWrite(LedPin, LOW);
    }
  }

  if(tm.checkTimeup()) {    // 130530
    // 16ms周期
    Count++;
    if(Count == 125) {
      // 1s周期
      Count = 0;
      if(LedSts) {
        // CAN送信バッファへ送信メッセージ、データを設定
        Can.SetTxBuf(0, 0x201, dat, 0, true);
        Can.TxReq(0);
      } else {
        Can.SetTxBuf(0, 0x200, dat, 0, true);
        Can.TxReq(0);
      }
      LedSts = !LedSts;
    }
  }
  if(Can.CheckTxComp(0)) {
    // 送信完了チェック
  }
}
