/*
  SDカード ファイル 書き込み + #129 SPI 表示
    copyright (c) 2013 T.Nakao      http://www.wsnak.com

    120515

  SD card I/F接続
    MOSI (D11)
    MISO (D12)
    CLK  (D13)
    CS   (D10)

  #129 SPI 6桁7セグメントLED
    MOSI (D11)
    MISO (D12)
    CLK  (D13)
    CS   (D9)
*/

#include <SD.h>       // SDライブラリ
#include <SPI.h>

#define SPI_CS_ON   digitalWrite(SPI_cs, LOW)      // #129 /cs = L
#define SPI_CS_OFF  digitalWrite(SPI_cs, HIGH)     // #129 /cs = H

const int SD_cs = 10;    // SDカード CSポート番号
const int SPI_cs = 9;    // #129 CSポート番号

int Count = 0;

File myFile;        // ファイルハンドル

void setup()
{
  pinMode(SPI_cs, OUTPUT);   // #129 cs
  pinMode(SD_cs, OUTPUT);    // SPI CS 出力設定
  SPI_CS_OFF;

  Serial.begin(19200);            // シリアル初期化

  SPI.begin();
  SPI.setBitOrder(MSBFIRST);      // 送信ビット順番

  if (!SD.begin(SD_cs)) {    // SDオブジェクト初期化
    Serial.println("initialization failed!"); // 初期化失敗
  } else {
    Serial.println("initialization done.");   // 初期化成功
  }
}


void loop() {
  // ファイルを書き込みでオープン
  myFile = SD.open("sd_129.txt", FILE_WRITE);
  if (myFile) {
    Count++;
    myFile.println(Count);    // ファイル書き込み
    myFile.close();           // ファイルクローズ

    Serial.println("write done.");

    DispHexa(Count);    // #129表示
  } else {
    // ファイルオープンエラー
    Serial.println("error opening test.txt");
  }
  delay(1000);
}


// SPIデータライト
void SpiSend(byte dat) {
  SPI.transfer(dat);
  delayMicroseconds(50);    // 50us
}


// #129へ16進4桁表示 (SPI送信)
void DispHexa(int num) {
  int dat;
  byte dg, i;
  dat = num;

  SPI.setDataMode(SPI_MODE1);   // SPIデータモード切替(for #129)
  SPI_CS_ON;

  SpiSend(0x40 + 5);            // CP=5(右端)
  for(i = 0; i < 4; i++) {      // 下位から順番に1桁ずつ送信
    dg = (byte)(dat & 0x0F);    // 下位4ビット(HEXA1桁分)
    SpiSend(0x00 + 0x10 + dg);  // 数値表示(CP左移動)
    dat = dat >> 4;             // 次の表示桁
  }

  SPI_CS_OFF;
  SPI.setDataMode(SPI_MODE0);   // SPIデータモード切替(for SD)
}
