//
//  32時間制クロック
//    copyright (c) 2013 T.Nakao      http://www.wsnak.com
//
//  12/03/02
//  12/03/11 時計微調整 試作
//
/*

  スイッチ配置
           ●SW1(D3)
    ●SW4(D6)    ● SW2(D4)
           ●SW3(D5)

  SW1 UP
  SW3 DOWN
  SW2 MODE
  SW4 SEL
  LED アラーム(D2)

*/


#include <wSwitch.h>        // SW
#include <wCTimer.h>        // タイマ
#include <LiquidCrystal.h>  // LCD

#define ON HIGH
#define OFF LOW

// LCDピンアサイン
#define LCD_RS 11       // D11
#define LCD_E  12       // D12
#define LCD_D4 7        // D7
#define LCD_D5 8        // D8
#define LCD_D6 9        // D9
#define LCD_D7 10       // D10

wCtcTimer2A tm;         // タイマオブジェクトのインスタンス
w4Switch sw;            // スイッチオブジェクトのインスタンス
// LCDのインスタンス化
LiquidCrystal lcd(LCD_RS, LCD_E, LCD_D4, LCD_D5, LCD_D6, LCD_D7);

char StrBuf[16];
int SubCnt = 0;       // 1秒サブカウンタ

byte Hour, Min, Sec, TmOfs; // カレント時刻
byte AlmHour, AlmMin;         // アラーム時刻
byte OfsHour;
byte Modify = false;
byte stHour, stMin, stSec;  // 設定用カレント時刻バッファ

byte BlkSts = false;
byte AlmSw = false;
byte AlmOn = false;

byte NormMode = true;         // 通常モード/設定モード
byte Mode = 0;              // 0:

byte LedSts = false;

/*
  Mode
  0:    通常表示
  1:    通常モードのアラームON/OFF切替
  2:    通常モードのアラーム設定[時]
  3:    通常モードのアラーム設定[分]
  11;   設定モードの現在時刻設定[時]
  12:   設定モードの現在時刻設定[分]
  13:   設定モードの現在時刻設定[秒]
  14:   設定モードの現在時刻設定[オフセット時]

*/

// 120311 時計微調整関係
int AdjCnt = 0;       // 補正周期用カウンタ
byte Adj = 1;         // 補正フラグ 0:補正なし 1:遅らせる 2:進める
int MaxAdjCnt = 74;   // 補正周期N (N回に1回補正)

// 初期化
void setup(void) {

  // analogReference(INTERNAL);     // 内部リファレンス(1.1V)使用

  pinMode(13, OUTPUT);              // LED

  tm.init(125);                     // タイマ値 8ms

  sw.initSwitch(3, 4, 5, 6);        // SW用ポート定義
  sw.initLed(2, 2);                 // LED用ポート定義

  sw.onKeyPress(KeyPress);          // スイッチ押下時のハンドラ登録

  lcd.begin(16, 2);                 // LCDを16文字2列に設定

  Hour = 0;
  Min = 0;
  Sec = 0;
  TmOfs = 7;

  AlmHour = 24;
  AlmMin = 30;
  AlmSw =  true;
}



void loop(void) {
  int adval;
  int val;

    // タイマ周期処理
  if(tm.checkTimeup()) {
    SubCnt++;
    sw.keyProc();                   // キーセンス処理 8ms周期

    if(SubCnt == 123) {             // 8ms * 125 = 1000ms

      // ----------------------------
      // 1秒周期処理
      // ----------------------------
        Sec++;
        //Sec+= 30;
        if(Sec == 60) {
          Min++;
          // Min+= 30;
          Sec = 0;
          if(Min == 60) {
            Hour++;
            Min = 0;
            if(Hour == 24) {
              Hour = 0;
            }
          }
        }

        // オンボードLED点滅
        if(LedSts) {
          digitalWrite(13, LOW);    // LED OFF
        } else {
          digitalWrite(13, HIGH);   // LED ON
        }
        LedSts = !LedSts;     // LED点消灯状態反転

      // ----------------------------
      // 現在時刻表示
      // ----------------------------
      if(Mode == 0) {
        // LCD表示 現在時刻
        sprintf(StrBuf, "%02d:%02d:%02d", Hour, Min, Sec);
        lcd.setCursor(0, 0);        // 表示位置を指定
        lcd.print(StrBuf);          // LCDに文字列を表示
      }
      return;
    }

    // ----------------------------
    // オフセット付き時刻表示
    // ----------------------------
    if(SubCnt == 124) {             // 8ms * 125 = 1000ms
      if(Mode == 0) {
        // LCD表示 オフセット現在時刻
        OfsHour = Hour;
        if(TmOfs > 0) {
          if(Hour <= TmOfs) {
            OfsHour = Hour + 24;
          }
        }
        sprintf(StrBuf, "%02d:%02d:%02d", OfsHour, Min, Sec);
        lcd.setCursor(0, 1);        // 表示位置を指定
        lcd.print(StrBuf);          // LCDに文字列を表示
      }
      return;
    }

    // ----------------------------
    // アラーム時刻表示
    // ----------------------------
    if(SubCnt == 125) {             // 8ms * 125 = 1000ms

      // 120311 ADJ 時計微調整(次の1秒周期を補正)
      AdjCnt++;
      if(AdjCnt == MaxAdjCnt) {
        // 補正タイミング
        AdjCnt = 0;
        if(Adj == 0) {
          SubCnt = 0;
        } else if(Adj == 1) {
          SubCnt = -1;          // 遅らせる(カウンタ値+1=126)
        } else {
          SubCnt = 1;           // 進ませる(カウンタ値-1=124)
        }
      } else {
        // 非補正タイミング
        SubCnt = 0;
      }

      if(Mode == 0) {
        // LCD表示 アラーム時刻
        char flg;
        if(AlmSw) {
          flg = 'A';
        } else {
          flg = ' ';
        }
        sprintf(StrBuf, "%c %02d:%02d", flg, AlmHour, AlmMin);
        lcd.setCursor(9, 0);        // 表示位置を指定

        lcd.print(StrBuf);          // LCDに文字列を表示
      }

      // アラーム時刻チェック
      byte hour;
      if(AlmHour > 23) {
        hour = AlmHour - 24;
      } else {
        hour = AlmHour;
      }
      if(AlmSw) {
        if(Hour == hour) {
          if(Min == AlmMin) {
            AlmOn = true;
            BlkSts =false;
          } else {
            AlmOn = false;
          }
        }
      }
    }

    // アラームLED点滅処理
    //   8ms * 32 = 256ms周期
    if(AlmOn) {
      if((SubCnt & 0x1F) == 1) {
        if(BlkSts) {
          sw.led1(OFF);
        } else {
          sw.led1(ON);
        }
        BlkSts = !BlkSts;     // LED点消灯状態反転
      }
    }
  }
}


//
// キー入力ハンドラ
//   スイッチが押されたときに呼び出される処理
//
void KeyPress(byte keyval) {
  switch(keyval) {
    case 1:     // UP
      KeyProcUp();
      break;
    case 2:     // SEL
      KeyProcSel();
      break;
    case 3:     // DOWN
      KeyProcDown();
      break;
    case 4:     // MODE
      KeyProcMode();
      break;
  }
}

// MODEキー
void KeyProcMode(void) {
  if(!NormMode) {
    Mode = 0;             // 通常の時刻表示

    if(Modify) {
      Modify = false;
      // 変更を反映
      Hour= stHour;
      Min = stMin;
      Sec = stSec;
      SubCnt = 0;
    }
  } else {
    Mode = 11;              // 設定モードの現在[時]設定画面
    Modify = false;
    // 現在時刻を保存
    stHour= Hour;
    stMin = Min;
    stSec = Sec;
  }
  NormMode = !NormMode;       // 反転
  MenuDisp();
}

// SELキー
void KeyProcSel(void) {
  if(Mode < 4) {
    // ノーマルモード 0, 1, 2, 3, 0, ...
    Mode++;
    if(Mode == 4) {
      Mode = 0;
    }
  } else {
    // 設定モード 11, 12, 13, 14, 11, ...
    Mode++;
    if(Mode == 15) {
      Mode = 11;
    }
  }
  MenuDisp();
}


void MenuDisp(void) {
  lcd.clear();

  switch(Mode) {
    case 0:     // 通常表示
//      lcd.noCursor();
      return;
    case 1:     // 通常モードのアラームON/OFF切替
      lcd.print("ALM SET:");
      if(AlmSw) {
        lcd.print("ON ");
      } else {
        lcd.print("OFF");
      }
      lcd.setCursor(10, 0);
//      lcd.cursor();
      return;
    case 2:     // 通常モードのアラーム設定[時]
      lcd.print("ALM Hur:");
      sprintf(StrBuf, "%02d", AlmHour);
      lcd.print(StrBuf);
      break;
    case 3:     // 通常モードのアラーム設定[分]
      lcd.print("ALM Min:");
      sprintf(StrBuf, "%02d", AlmMin);
      lcd.print(StrBuf);
      break;
    case 11:    // 設定モードの現在時刻設定[時]
      lcd.print("SET Hur:");
      sprintf(StrBuf, "%02d", stHour);
      lcd.print(StrBuf);
      break;
    case 12:    // 設定モードの現在時刻設定[分]
      lcd.print("SET Min:");
      sprintf(StrBuf, "%02d", stMin);
      lcd.print(StrBuf);
      break;
    case 13:    // 設定モードの現在時刻設定[秒]
      lcd.print("SET Sec:");
      sprintf(StrBuf, "%02d", stSec);
      lcd.print(StrBuf);
      break;
    case 14:    // 設定モードの現在時刻設定[オフセット時]
      lcd.print("SET Ofs:");
      sprintf(StrBuf, "%02d", TmOfs);
      lcd.print(StrBuf);
      break;
  }
  lcd.setCursor(9, 0);
//  lcd.cursor();
}


// UPキー
void KeyProcUp(void) {
  switch(Mode) {
    case 0:     // 通常表示
      break;
    case 1:     // 通常モードのアラームON/OFF切替
      AlmSw = !AlmSw;
      if(!AlmSw) {
        AlmOff();
      }
      break;
    case 2:     // 通常モードのアラーム設定[時]
      if(AlmHour < (24 + TmOfs)) {
        AlmHour++;
      } else {
        AlmHour = 0;
      }
      break;
    case 3:     // 通常モードのアラーム設定[分]
      if(AlmMin < 59) {
        AlmMin++;
      } else {
        AlmMin = 0;
      }
      break;
    case 11:    // 設定モードの現在時刻設定[時]
      if(stHour < 23) {
        stHour++;
      } else {
        stHour = 0;
      }
      Modify = true;
      break;
    case 12:    // 設定モードの現在時刻設定[分]
      if(stMin < 59) {
        stMin++;
      } else {
        stMin = 0;
      }
      Modify = true;
      break;
    case 13:    // 設定モードの現在時刻設定[秒]
      if(stSec < 59) {
        stSec++;
      } else {
        stSec = 0;
      }
      Modify = true;
      break;
    case 14:    // 設定モードの現在時刻設定[オフセット時]
      if(TmOfs < 8) {
        TmOfs++;
      } else {
        TmOfs = 0;
      }
      break;
  }
  MenuDisp();
}


// Downキー
void KeyProcDown(void) {
  switch(Mode) {
    case 0:     // 通常表示
      break;
    case 1:     // 通常モードのアラームON/OFF切替
      AlmSw = !AlmSw;
      if(!AlmSw) {
        AlmOff();
      }
      break;
    case 2:     // 通常モードのアラーム設定[時]
      if(AlmHour > 0) {
        AlmHour--;
      } else {
        AlmHour = 24 + TmOfs;
      }
      break;
    case 3:     // 通常モードのアラーム設定[分]
      if(AlmMin > 0) {
        AlmMin--;
      } else {
        AlmMin = 59;
      }
      Modify = true;
      break;
    case 11:    // 設定モードの現在時刻設定[時]
      if(stHour > 0) {
        stHour--;
      } else {
        stHour = 23;
      }
      Modify = true;
      break;
    case 12:    // 設定モードの現在時刻設定[分]
      if(stMin > 0) {
        stMin--;
      } else {
        stMin = 59;
      }
      Modify = true;
      break;
    case 13:    // 設定モードの現在時刻設定[秒]
      if(stSec > 0) {
        stSec--;
      } else {
        stSec = 59;
      }
      Modify = true;
      break;
    case 14:    // 設定モードの現在時刻設定[オフセット時]
      if(TmOfs > 0) {
        TmOfs--;
      } else {
        TmOfs = 8;
      }
      break;
  }
  MenuDisp();
}


// アラーム停止
void AlmOff(void) {
  AlmOn = false;
  sw.led1(OFF);
}
