//
//  温度読み出し、SDカード記録
//    copyright (c) 2013 T.Nakao      http://www.wsnak.com
//
//  A-Dコンバータより温度を読み出して換算. その温度値をSDカードへ保存(開始時ファイルオープン、終了時ファイルクローズ)
//  120229
//

/*
  SD card I/F接続
  MOSI (D11)
  MISO (D12)
  CLK (D13)
  CS (D10)
*/

#include <wCTimer.h>        // タイマ
#include <SD.h>             // SDライブラリ
#include <wSwitch.h>       // SW

#define ON HIGH
#define OFF LOW

wCtcTimer2A tm;         // タイマオブジェクトのインスタンス
w4Switch sw;            // スイッチオブジェクトのインスタンス

float ThTmp;            // 摂氏温度
float Factor = 0.488;   // 換算係数 Ref = 5V

#define SPI_CS  10  // CSポート番号

File myFile;        // ファイルハンドル

char StrBuf[16];
int SubCnt = 0;        // 1秒サブカウンタ
int RecNum = 0;       // 記録レコード連番
byte inRec = false;   // 記録中フラグ
byte RecSw = false;   // スイッチオルタネートフラグ
byte LedSts = false;  // LEDオルタネートフラグ

// 初期化
void setup(void) {
  pinMode(SPI_CS, OUTPUT);          // SPI CS 出力設定

  // analogReference(EXTERNAL);     // 外部リファレンス(ARef)使用
  // analogReference(INTERNAL);     // 内部リファレンス(1.1V)使用

  if (!SD.begin(SPI_CS)) {      // SDオブジェクト初期化
    // 初期化失敗
    return;
  }

  tm.init(125);                     // タイマ値 8ms
  sw.initSwitch(2);                 // SW用ポート定義
  sw.initLed(3, 3);                 // LED用ポート定義

  sw.onKeyPress(KeyPress);          // スイッチ押下時のハンドラ登録

  inRec = false;
  RecSw = false;
  LedSts = false;
}


// メインループ
void loop(void) {
  int adval;
  int val;


    // タイマ周期処理
  if(tm.checkTimeup()) {
    SubCnt++;
    sw.keyProc();                   // キーセンス処理 8ms周期

    if(SubCnt >= 125) {              // 8ms * 125 = 1000ms
      SubCnt = 0;

      // ----------------------------
      // 1秒周期処理
      if(inRec) {
        // 記録中

        // 温度測定
        adval = analogRead(A0);       // 熱電対電圧のA-D変換値の読み出し
        ThTmp = adval * Factor;       // 摂氏温度に換算

        val = (int)(ThTmp * 10);      // 10倍して整数に変換

        // SDカードに書き込み
        if(myFile) {
          // ファイル書き込み
          sprintf(StrBuf, "%05d:%d.%d", RecNum, val / 10, val % 10);
          myFile.println(StrBuf);     // 書き込み
          RecNum++;                   // レコード連番更新
        }
      }
      // ----------------------------

    }
  }
}


// 記録開始
void BeginRec(void) {
  SD.begin(SPI_CS);      // SDオブジェクト初期化(カード抜いた後の再挿入対策)

  myFile = SD.open("data.txt", FILE_WRITE); // ファイルオープン
  if(myFile) {
    // オープン成功
    inRec = true;
    RecNum = 0;       // レコード連番リセット
    sw.led1(ON);
  }
}


// 記録停止
void EndRec(void) {
  if(myFile) {
    inRec = false;
    myFile.close();   // ファイルクローズ
    sw.led1(OFF);
  }
}


//
// キー入力ハンドラ
//
void KeyPress(byte keyval) {
  if(keyval == 1) {
    if(!RecSw) {
      BeginRec();     // 記録開始
    } else {
      EndRec();       // 記録停止
    }
    RecSw = !RecSw;   // SW状態オルタネート
  }
}
