//
//  ノードA
//  A-Dコンバータより温度を読み出して換算. その温度値をCAN送信
//    copyright (c) 2013 T.Nakao      http://www.wsnak.com
//
//  12/02/27
//

#include <wCan2515.h>
#include <p2515Reg.h>
#include <wCTimer.h>    // タイマ

#define TmpCmd  1   // CAN 温度通知コマンド

// CANドライバのインスタンス生成
wCan2515 Can(2, 4, 3, 5, CAN_BRP_16MHz_125KBPS);
// D2:SCK, D3:SO, D4:SI, D5:SS (SI/SOクロス)

wCtcTimer2A tm;         // タイマのインスタンス

byte MyID;              // 温度計ID(0～15)
float ThTmp;            // 摂氏温度
float Factor = 0.488;   // 換算係数 Ref = 5V

#define CANLED_ON Can.BitModCmd(BFPCTRL, (1<<B0BFS) | (1<<B0BFE), (1<<B0BFS) | (1<<B0BFE))
#define CANLED_OFF  Can.BitModCmd(BFPCTRL, (1<<B0BFS) | (1<<B0BFE), 1<<B0BFE)

#define LED_PIN 13                            // on board LED
#define LED_ON  digitalWrite(LED_PIN, HIGH)
#define LED_OFF digitalWrite(LED_PIN, LOW)

// 温度計IDを設定用ジャンパポート
#define JP1 6   // D0
#define JP2 7   // D7

// 初期化
void setup(void) {

  pinMode(LED_PIN, OUTPUT);
  pinMode(JP1, INPUT);                    // 温度計ID設定用ジャンパ
  pinMode(JP2, INPUT);                    //

	// analogReference(EXTERNAL);						// 外部リファレンス(ARef)使用
	// analogReference(INTERNAL);						// 内部リファレンス(1.1V)使用

  // CANコントローラ設定
  Can.SetMask(MASK_SID_ALL_HIT);          // マスクなし
  Can.SetFilter(0, FILTER_MOD_ALL_HIT);   // フィルタ全メッセージ
  Can.SetOpMode(CAM_MODE_NORMAL);         // ノーマルモードに切替

  // CANコントローラ SPI送信テスト
  CANLED_ON;    // LED ON
  delay(500);
  CANLED_OFF;   // LED OFF
  delay(500);
  CANLED_ON;    // LED ON

  tm.init(125); // タイマ値 8ms (x125=1000ms)

  // JP1, JP2の状態読み出し温度計IDを設定 ショートで0, オープンで1
  MyID = 0;
  if(digitalRead(JP1) == HIGH) {
    MyID |= 1;                      // b0
  }
  if(digitalRead(JP2) == HIGH) {
    MyID |= 2;                      // b1
  }
}

int Count = 0;

// メインループ
void loop(void) {
  byte size;
  unsigned int msg;     // CANメッセージ
  byte dat[8];

  int adval;
  int val;

  // タイマ周期処理
  if(tm.checkTimeup()) {
    Count++;
    if(Count >= 125) {    // 8ms * 125 = 1000ms
      // 1秒周期処理
      Count = 0;

      // 温度測定
      adval = analogRead(A0);       // 熱電対電圧のA-D変換値の読み出し
      ThTmp = adval * Factor + 0.5; // 摂氏温度に換算, 小数点以下四捨五入

      // CAN通信用データを作る
      msg = MyID | (TmpCmd << 4);   // CANメッセージ(SID)
      val = (int)ThTmp;             // 摂氏温度の整数部

      dat[0] = val >> 8;            // 上位8ビット
      dat[1] = (byte)val;           // 下位8ビット
      size = 2;                     // CAN送信データ数

      // CAN送信
      Can.SetTxBuf(0, msg, dat, size, true);  // CAN送信バッファへ送信メッセージ、データを設定
      Can.TxReq(0);                 // 送信要求発行
      LED_ON;                       // LED ON(送信要求)
    }
  }

  // 送信完了チェック (注.この処理は削除してはいけない)
  if(Can.CheckTxComp(0)) {
    // 送信完了時の処理
    LED_OFF;                        // LED OFF(送信完了)
  }
}
