//
//  ノードB
//  CAN受信した摂氏温度をシリアルでPCへ送信する
//    copyright (c) 2013 T.Nakao      http://www.wsnak.com
//
//  12/02/27
//

#include <wCan2515.h>           // CANドライバ
#include <p2515Reg.h>           // CAN関連レジスタ定義

// CANドライバのインスタンス生成
wCan2515 Can(2, 4, 3, 5, CAN_BRP_16MHz_125KBPS);
// D2:SCK, D3:SO, D4:SI, D5:SS (SI/SOクロス)

#define CANLED_ON Can.BitModCmd(BFPCTRL, (1<<B0BFS) | (1<<B0BFE), (1<<B0BFS) | (1<<B0BFE))
#define CANLED_OFF  Can.BitModCmd(BFPCTRL, (1<<B0BFS) | (1<<B0BFE), 1<<B0BFE)

#define LED_PIN 13                            // on board LED
#define LED_ON  digitalWrite(LED_PIN, HIGH)
#define LED_OFF digitalWrite(LED_PIN, LOW)


// 初期化
void setup(void) {
  // シリアル通信パラメータ設定
  Serial.begin(19200);

  // CANコントローラ設定
  Can.setMask(MASK_SID_ALL_HIT);          // マスクなし
  Can.setFilter(0, FILTER_MOD_ALL_HIT);   // フィルタ全メッセージ
  Can.setOpMode(CAM_MODE_NORMAL);         // ノーマルモードに切替

  // CANコントローラ SPI送信テスト
  CANLED_ON;    // LED ON
  delay(500);
  CANLED_OFF;   // LED OFF
  delay(500);
  CANLED_ON;    // LED ON
}

// メインループ
void loop(void) {
  byte size, datfrm;
  unsigned int msg;
  byte dat[8];        // CANメッセージ用バッファ

  int val;
  char buf[10];       // 文字列生成用バッファ

  byte id;

  // CAN受信処理
  if(Can.rxCheck()) { // CAN受信チェック
    // CANメッセージを受信しているとき
    msg = Can.GetRxBuf(0, dat, &size, &datfrm);   // データとメッセージを取り出す。

    id = msg & 0x0F;  // 温度計ID

    // 摂氏温度の取り出し(整数値)
    val = (int)dat[0] << 8;
    val |= (int)dat[1];

    // 摂氏温度送信
    sprintf(buf, "%2d:%3d\r\n", id, val);
    Serial.print(buf);                  // シリアル出力
  }
}

