/*
  放電器 4本並列
    copyright (c) 2013 T.Nakao    http://www.wsnak.com

    12/04/05
      SDカード記録
      LCD表示
      測定周期 5秒周期

      12/04/05 swスタート


  SD card I/F接続
  MOSI (D11)
  MISO (D12)
  CLK (D13)
  CS (D10)

  AREF 2.0V
  2000 / 1024 = 1.953mV/bit

  ch1 D2/A0
  ch2 D3/A1
  ch3 D4/A2
  ch4 D5/A3
*/

#include <SD.h>
#include <wCTimer.h>        // タイマ
#include <LiquidCrystal.h>  // LCD
#include <wSwitch.h>        // SW A4,A5

#define ON HIGH
#define OFF LOW

#define STOP_VOL 1000     // 放電停止電圧(mV)

// LCDピンアサイン
#define LCD_RS 0
#define LCD_E  1
#define LCD_D4 6
#define LCD_D5 7
#define LCD_D6 8
#define LCD_D7 9

// 放電切り換えポート
#define Ch1On   2     // D2
#define Ch2On   3     // D3
#define Ch3On   4     // D4
#define Ch4On   5     // D5

#define DisChgOn1 digitalWrite(Ch1On, HIGH)     // 放電開始
#define DisChgOn2 digitalWrite(Ch2On, HIGH)     // 放電開始
#define DisChgOn3 digitalWrite(Ch3On, HIGH)     // 放電開始
#define DisChgOn4 digitalWrite(Ch4On, HIGH)     // 放電開始

#define DisChgOff1  digitalWrite(Ch1On, LOW)    // 放電停止
#define DisChgOff2  digitalWrite(Ch2On, LOW)    // 放電停止
#define DisChgOff3  digitalWrite(Ch3On, LOW)    // 放電停止
#define DisChgOff4  digitalWrite(Ch4On, LOW)    // 放電停止


// LCDのインスタンス化
LiquidCrystal lcd(LCD_RS, LCD_E, LCD_D4, LCD_D5, LCD_D6, LCD_D7);

// A4,A5スイッチ入力
w2SwitchA4A5 Sw;        // スイッチオブジェクトのインスタンス

const int chipSelect = 10;    // CSポート番号

File myFile;
wCtcTimer2A tm;         // タイマオブジェクトのインスタンス
//w4Switch sw;            // スイッチオブジェクトのインスタンス

char StrBuf[16];
int SubCnt = 0;        // 1秒サブカウンタ

float ThTmp;            // 摂氏温度
float Factor = 1.953;   // 換算係数 Ref = 2V

byte inDisCrg = false;  // 放電中フラグ


// -------------------------
// 初期化
// -------------------------
void setup()
{
  analogReference(EXTERNAL);     // 外部リファレンス(ARef)使用

  pinMode(chipSelect, OUTPUT);     // SPI CS 出力設定
  if (!SD.begin(chipSelect)) {
//    return;
  }

  tm.init(125);                     // タイマ値 8ms
  lcd.begin(16, 2);                 // LCDを16文字2列に設定

  pinMode(Ch1On, OUTPUT);           // TR 出力設定
  pinMode(Ch2On, OUTPUT);           // TR 出力設定
  pinMode(Ch3On, OUTPUT);           // TR 出力設定
  pinMode(Ch4On, OUTPUT);           // TR 出力設定

  Sw.init();                        // スイッチ入力(A4,A5)
  Sw.onKeyPress(KeyPress);          // スイッチ入力ハンドラ登録

}


// -------------------------
// メインループ
// -------------------------
void loop() {
  int adval;
  int val[4];
  byte i;

    // タイマ周期処理
  if(tm.checkTimeup()) {
    SubCnt++;

    Sw.keyProc();

    if(SubCnt >= 250) {              // 8ms * 250 = 2000ms
//    if(SubCnt >= 625) {              // 8ms * 625 = 5000ms
      SubCnt = 0;
      // ----------------------------
      // 1秒周期処理
      // ----------------------------

      // 電圧測定、停止判定
      adval = analogRead(A0);       // 電圧のA-D変換値の読み出し
      ThTmp = adval * Factor;       // 電圧値に換算
      val[0] = (int)ThTmp;          // 整数(mV)に変換
      if(val[0] <= STOP_VOL) {      // 放電終了電圧判定
        DisChgOff1;                 // 放電OFF
      }

      adval = analogRead(A1);       // 電圧のA-D変換値の読み出し
      ThTmp = adval * Factor;       // 電圧値に換算
      val[1] = (int)ThTmp;          // 整数(mV)に変換
      if(val[1] <= STOP_VOL) {      // 放電終了電圧判定
        DisChgOff2;                 // 放電OFF
      }

      adval = analogRead(A2);       // 電圧のA-D変換値の読み出し
      ThTmp = adval * Factor;       // 電圧値に換算
      val[2] = (int)ThTmp;          // 整数(mV)に変換
      if(val[2] <= STOP_VOL) {      // 放電終了電圧判定
        DisChgOff3;                 // 放電OFF
      }

      adval = analogRead(A3);       // 電圧のA-D変換値の読み出し
      ThTmp = adval * Factor;       // 電圧値に換算
      val[3] = (int)ThTmp;          // 整数(mV)に変換
      if(val[3] <= STOP_VOL) {      // 放電終了電圧判定
        DisChgOff4;                 // 放電OFF
      }

      // 放電動作中のときだけSDカードへ保存
      if(inDisCrg) {
        // 放電中のとき
        // SDカード書き込み
        myFile = SD.open("Vollog.txt", FILE_WRITE);
        if (myFile) {
          for(i = 0; i < 3; i++) {
            sprintf(StrBuf, "%d\t", val[i]);
            // ファイル書き込み
            myFile.print(StrBuf);
          }
          sprintf(StrBuf, "%d", val[3]);
          myFile.println(StrBuf);

          // ファイルクローズ
          myFile.close();
        }
      }

      // LCDへ表示
      lcd.setCursor(0, 0);        // 表示位置を指定
      sprintf(StrBuf, "1:%4d 3:%4d", val[0], val[2]);
      lcd.print(StrBuf);          // LCDに文字列を表示

      lcd.setCursor(0, 1);        // 表示位置を指定
      sprintf(StrBuf, "2:%4d 4:%4d", val[1], val[3]);
      lcd.print(StrBuf);          // LCDに文字列を表示
    }
  }
}


// -------------------------
// スイッチ入力処理
// -------------------------
void KeyPress(byte keyval) {
  switch(keyval) {
    case 1:     // SW1 放電開始
      DisChgOn1;
      DisChgOn2;
      DisChgOn3;
      DisChgOn4;
      inDisCrg = true;    // 放電中

      if (!SD.begin(chipSelect)) {    // SDカード オープン
      //    return;
      }
      break;

    case 2:     // SW2 放電停止
      DisChgOff1;
      DisChgOff2;
      DisChgOff3;
      DisChgOff4;
      inDisCrg = false;   // 停止中
      break;
  }
}
