//
//  I2C制御 LCDコントローラ
//    copyright (c) 2013 T.Nakao      http://www.wsnak.com
//
//    汎用LCDモジュールのI2C化キット
//
//    12/11/15 (c)T.Nakao
//

/*
  汎用のLCDへ接続して、I2Cデバイス化する、Arduino使用のコントローラ
*/

#include <Wire.h>

/*
  LCD data PORTD D4～D7
  LCD E  D8
  LCD RS D9
*/

#define E_ON    digitalWrite(8, HIGH)
#define E_OFF   digitalWrite(8, LOW)
#define RS_CMD  digitalWrite(9, LOW)    // RS=0
#define RS_DATA digitalWrite(9, HIGH)   // RS=1

#define LCD_DAT PORTD     // LCD DB0～DB7

#define TST_SW  !(PINC&(1<<3))          // PC3 (A3) ジャンパショート時テスト

byte CmdCnt;
byte Cmd, Data;

// プロトタイプ
void writeCmd(void);
void WriteData(byte dat);
void WriteCmd(byte dat);
void Write4bitCmd(byte data);
void Write4bitData(byte data);
void LcdInit(void);
void E_Pulse(void);


void setup(void) {
  // 入力ポートのデジタルバッファをイネーブルにする
  DIDR0 &= ~(1<<ADC3D);   // ADC3

  // A3(PC3) ディジタル入力に再設定
  DDRC &= ~(1<<3);      // input
  PORTC |= (1<<3);      // pullup enable

//  Wire.begin(0x50);
  Wire.begin(0x20);
  Wire.onReceive(hSlvRcv);

  CmdCnt = 0;

  pinMode(4, OUTPUT);   // LCD DB4
  pinMode(5, OUTPUT);
  pinMode(6, OUTPUT);
  pinMode(7, OUTPUT);   // LCD DB7


  pinMode(8, OUTPUT);   // LCD E
  pinMode(9, OUTPUT);   // LCD RS

  E_OFF;
  RS_CMD;

  LcdInit();

  // テスト表示(A3ポートのジャンパ設定による)
  if(TST_SW) {
    delay(100);
    WriteData('T');
    WriteData('E');
    WriteData('S');
    WriteData('T');
  }
}


// -----------------------
// メインループ
// -----------------------
void loop(void) {

}


// スレーブ受信ハンドラ
void hSlvRcv(int cnt) {
  int i;

  for(i = 0; i < cnt; i+=2) {
    Cmd = Wire.read();
    Data = Wire.read();
    writeCmd();
  }
}


// -----------------------
// コマンド書き込み
// -----------------------
void writeCmd(void) {
  if(Cmd == 0x00) {
    // LCDコマンド
    WriteCmd(Data);
  } else {
    // LCDデータ
    WriteData(Data);
  }

}


// *********************************
//  LCD初期化 (4ビットモード)
// *********************************
void LcdInit(void) {

  delay(15);

  Write4bitCmd(0x3);        //
  delay(5);
  Write4bitCmd(0x3);        //
  delay(5);
  Write4bitCmd(0x3);        //
  delay(5);
  Write4bitCmd(0x2);        //
  delay(5);

  WriteCmd(0x01);       // clear
  delay(5);

  WriteCmd(0x06);       // entry mode set
  delay(5);

  WriteCmd(0x0C);       // display on/off control
  delay(5);

  WriteCmd(0x1C);       // cursor/display shift
  delay(5);

  WriteCmd(0x28);       // function set
  delay(5);

  WriteCmd(0x02);       // home
  delay(5);
}


// LCDデータ書き込み
void WriteData(byte dat) {
  Write4bitData(dat >> 4);    // 上位4bit
  Write4bitData(dat);         // 下位4bit
}

// LCDコマンド書き込み
void WriteCmd(byte dat) {
  Write4bitCmd(dat >> 4);     // 上位4bit
  Write4bitCmd(dat);          // 下位4bit
}


// 4ビットLCDコマンド書き込み
void Write4bitCmd(byte data) {
  RS_CMD;

  LCD_DAT = data << 4;
  E_Pulse();
}

// 4ビットLCDデータ書き込み
void Write4bitData(byte data) {
  RS_DATA;

  LCD_DAT = data << 4;
  E_Pulse();
}

// eパルス出力
void E_Pulse(void) {
  E_OFF;
  delayMicroseconds(1);
  E_ON;
  delayMicroseconds(1);
  E_OFF;
  delayMicroseconds(100);
}
