//
//  I2C制御 LCDコントローラ(OPTREX C-5184NFJ-SLW-ADN)
//    copyright (c) 2013 T.Nakao      http://www.wsnak.com
//
//    40文字×4行 LCDモジュールのI2C化キット
//
//    12/11/16
//

/*
  OPTREX C-5184NFJ-SLW-ADN専用
  LCDへ接続して、I2Cデバイス化する、Arduino使用のコントローラ
*/


// 121116 (c)T.Nakao

#include <Wire.h>

char StrBuf[16];

/*
  LCD DATA PORTD2-7(D2-D7), PORTB0,1(D8,D9)
  LCD E1   D10
  LCD E2   D11
  LCD RS   D12
*/

#define E1_ON   digitalWrite(10, HIGH)
#define E1_OFF  digitalWrite(10, LOW)

#define E2_ON   digitalWrite(11, HIGH)
#define E2_OFF  digitalWrite(11, LOW)

#define RS_CMD  digitalWrite(12, LOW)   // RS=0
#define RS_DATA digitalWrite(12, HIGH)  // RS=1

#define LCD_DAT PORTD     // D0～D7

#define TST_SW  !(PINC&(1<<3))          // PC3 (A3) ジャンパショート時テスト

byte CmdCnt;
byte Cmd, Data;

byte SelArea;           // 1=E1, 2=E2 3=E1&E2 選択

void writeCmd(void);

// プロトタイプ
void WriteData(byte dat);
void WriteCmd(byte dat);
void POut8bit(byte dat);
void LcdInit(void);
void E_Pulse(void);


// -----------------------------------
// 初期化
// -----------------------------------
void setup(void) {
//  Serial.begin(19200);    // シリアル初期化

  // 入力ポートのデジタルバッファをイネーブルにする
  DIDR0 &= ~(1<<ADC3D);   // ADC3

  // A3(PC3) ディジタル入力に再設定
  DDRC &= ~(1<<3);      // input
  PORTC |= (1<<3);      // pullup enable

//  Wire.begin(0x50);
  Wire.begin(0x30);
  Wire.onReceive(hSlvRcv);

  CmdCnt = 0;

  pinMode(2, OUTPUT);   // LCD DB2
  pinMode(3, OUTPUT);   // LCD DB3
  pinMode(4, OUTPUT);   // LCD DB4
  pinMode(5, OUTPUT);   // LCD DB5
  pinMode(6, OUTPUT);   // LCD DB6
  pinMode(7, OUTPUT);   // LCD DB7
  pinMode(8, OUTPUT);   // LCD DB0 <- アサイン注意
  pinMode(9, OUTPUT);   // LCD DB1 <- アサイン注意

  pinMode(10, OUTPUT);    // LCD E1
  pinMode(11, OUTPUT);    // LCD E2
  pinMode(12, OUTPUT);    // LCD RS

  E1_OFF;
  RS_CMD;

  LcdInit();

  if(TST_SW) {
    delay(500);

    SelArea = 0;            // E1
    WriteData('A');
    WriteData('R');
    WriteData('E');
    WriteData('A');
    WriteData('1');
    WriteData(' ');

    SelArea = 1;            // E2
    WriteData('A');
    WriteData('R');
    WriteData('E');
    WriteData('A');
    WriteData('2');
    WriteData(' ');

    SelArea = 2;            // E1,E2
    WriteData('A');
    WriteData('R');
    WriteData('E');
    WriteData('A');
    WriteData('1');
    WriteData('&');
    WriteData('2');
  }

}


// -----------------------------------
// メインループ
// -----------------------------------
void loop(void) {

}


// -----------------------------------
// スレーブ受信ハンドラ
// -----------------------------------
void hSlvRcv(int cnt) {
  int i;

  for(i = 0; i < cnt; i+=2) {
    Cmd = Wire.read();
    Data = Wire.read();
    writeCmd();
  }
}


// -----------------------------------
// コマンド書き込み(I2Cコマンド処理)
// -----------------------------------
void writeCmd(void) {

  switch(Cmd) {
    case 0x00:        // LCDコマンド
      WriteCmd(Data);
      break;
    case 0x80:        // LCDデータ
      WriteData(Data);
      break;

    case 0xFF:        // エリア設定コマンド
      SelArea = Data;
      break;
  }
}


// -----------------------------------
//  LCD初期化 (8ビットモード)
// -----------------------------------
void LcdInit(void) {

  SelArea = 2;          // E1,E2

  delay(15);

  WriteCmd(0x38);   // function set
  delay(5);
  WriteCmd(0x38);   //
  delay(5);
  WriteCmd(0x38);   //
  delay(5);
  WriteCmd(0x38);   //
  delay(5);

  WriteCmd(0x01);       // clear
  delay(5);

  WriteCmd(0x06);       // entry mode set
  delay(5);

  WriteCmd(0x0C);       // display on/off control
  delay(5);

  WriteCmd(0x1C);       // cursor/display shift
  delay(5);

  WriteCmd(0x38);       // function set
  delay(5);

  WriteCmd(0x02);       // home
  delay(5);
}


// -----------------------------------
// LCDデータ ライト
// -----------------------------------
void WriteData(byte dat) {
  RS_DATA;

  POut8bit(dat);
  E_Pulse();
}


// -----------------------------------
// LCDコマンド ライト
// -----------------------------------
void WriteCmd(byte dat) {
  RS_CMD;

  POut8bit(dat);
  E_Pulse();
}


// -----------------------------------
// LCDデータポート 8ビット出力
// -----------------------------------
void POut8bit(byte dat) {
  byte  reg;

  // LCDデータ8ビット出力
  reg = PORTD & 0x03;         // RD0, RD1以外をマスク
  reg |= (dat & 0xFC);        // d7～d2をマージ
  PORTD = reg;                // 書き戻し

  reg = PORTB & 0xFC;         // RB7～RB2以外をマスク
  reg |= (dat & 0x03);        // d1,d0をマージ
  PORTB = reg;                // 書き戻し
}


// -----------------------------------
// Eパルス出力
// -----------------------------------
void E_Pulse(void) {

  E1_OFF;
  E2_OFF;
  delayMicroseconds(1);

  if(SelArea == 0) {
    // エリア0
    E1_ON;
  } else if(SelArea == 1) {
    // エリア1
    E2_ON;
  } else {
    // エリア0,1同時
    E1_ON;
    E2_ON;
  }

  delayMicroseconds(1);
  E1_OFF;
  E2_OFF;
  delayMicroseconds(100);
}

