/*
  熱電対コンバータ 温度計算テスト
    copyright (c) 2013 T.Nakao www.wsnak.com

  13/01/14 新規
*/


/*
  ******** 入力データ(SPI受信データの先頭から2バイト)
    +1600.00  0110 0100 0000 0000   0x6400
    +1000.00  0011 1110 1000 0000   0x3E80
    +100.75   0000 0110 0100 1100   0x064C
    +25.00    0000 0001 1001 0000   0x0190
    0.00      0000 0000 0000 0000   0x0000
    -0.25     1111 1111 1111 1100   0xFFFC
    -1.00     1111 1111 1111 0000   0xFFF0
    -250.00   1111 0000 0110 0000   0xF060

  ******** プログラム実行結果 **********
    +1600.0
    +1000.0
    +100.75
    +25.0
    +0.0
    -0.25
    -1.0
    -250.0
*/

char strBuf[16];

word tempDat[8] = {
  0x6400, // +1600.00 0110 0100 0000 0000
  0x3E80, // +1000.00 0011 1110 1000 0000
  0x064C, // +100.75    0000 0110 0100 1100
  0x0190, // +25.00   0000 0001 1001 0000
  0x0000, // 0.00     0000 0000 0000 0000
  0xFFFC, // -0.25      1111 1111 1111 1100
  0xFFF0, // -1.00      1111 1111 1111 0000
  0xF060  // -250.00    1111 0000 0110 0000
};


// -------------------------
// 初期化
// -------------------------
void setup()
{
  byte i;
  Serial.begin(19200);             // シリアルポート初期化

  for(i = 0; i < 8; i++) {
    getTemp(tempDat[i], strBuf);
    Serial.println(strBuf);
  }
}


// -------------------------
// メインループ
// -------------------------
void loop() {

}


void getTemp(word spival16, char *str) {
  word val, vali;
  byte sign, vals;

  val = spival16 >> 2;

  if(val & 0x2000) {  // 符号ビットをチェック
    // 負数(2の補数)
    val = ~val + 1;   // 正数に戻す
    val &= 0x3FFF;    // 数値抽出
    sign = true;    // 負数マーク
  } else {
    // 正数
    sign = false;
  }

  // 整数部分と小数部分に分ける
  vali = val >> 2;        // 整数部(上位11ビット)
  vals = (byte)val & 0x03;    // 小数部(下位2ビット)
  vals = vals * 25;       // 25=0.25*100 小数部を整数化

  // 表示用文字列作成
  if(sign) {
    // 負数
    sprintf(str, "-%d.%-2d", vali, vals);
  } else {
    // 整数
    sprintf(str, "+%d.%-2d", vali, vals);
  }
}

