//
//  温度読み出しテスト(LM35)
//    copyright (c) 2013 T.Nakao      http://www.wsnak.com
//
//  LM35で読み出した温度をLCDへ表示
//
//  12/02/29
//  13/05/30 関数名 誤記訂正
//

#include <wCTimer.h>    // タイマ

#include <LiquidCrystal.h>

#define LCD_RS 10         // D10
#define LCD_E 11           // D11
#define LCD_D4 6        // D6
#define LCD_D5 7        // D7
#define LCD_D6 8        // D8
#define LCD_D7 9        // D9

// LCDのインスタンス化
LiquidCrystal lcd(LCD_RS, LCD_E, LCD_D4, LCD_D5, LCD_D6, LCD_D7);

wCtcTimer2A tm;         // タイマのインスタンス

float ThTmp;            // 摂氏温度
float Factor = 0.1072;  // 換算係数 VRef = 1.1V LM35

#define LED_PIN 13                            // on board LED
#define LED_ON  digitalWrite(LED_PIN, HIGH)
#define LED_OFF digitalWrite(LED_PIN, LOW)

char StrBuf[16];

// 初期化
void setup(void) {

  pinMode(LED_PIN, OUTPUT);

  // analogReference(EXTERNAL);           // 外部リファレンス(ARef)使用
  analogReference(INTERNAL);            // 内部リファレンス(1.1V)使用

  tm.init(125); // タイマ値 8ms (x125=1000ms)

  lcd.begin(16, 2);    // LCDを16文字2列に設定

}

int Count = 0;

// メインループ
void loop(void) {
  byte size;
  unsigned int msg;     // CANメッセージ
  byte dat[8];

  int adval;
  int val;

  // タイマ周期処理
  if(tm.checkTimeup()) {
    Count++;
    if(Count >= 125) {    // 8ms * 125 = 1000ms
      // 1秒周期処理
      Count = 0;

      // 温度測定
      adval = analogRead(A0);       // 熱電対電圧のA-D変換値の読み出し
      ThTmp = adval * Factor + 0.5; // 摂氏温度に換算, 小数点以下四捨五入

      sprintf(StrBuf, "%2d", (int)(ThTmp * 10));
      lcd.setCursor(0, 1);    // 表示位置を指定
      lcd.print(StrBuf);    // LCDに文字列を表示
    }
  }
}
